/*
** Module   : HandleChain.java
** Abstract : An abstract class which provides an  implementation for the most
**            common methods from CommonHandleChain.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 CS  20130207 Created initial version.
*/

package com.goldencode.p2j.util;

/**
 * This acts as a superclass which implements the most common methods of the
 * CommonHandleChain interface:
 * <table>
 * <tr>
 *    <th>attribute or base method</th>
 *    <th>API</th>
 * </tr>
 * <tr>
 *    <td>NAME</td>
 *    <td>{@link #getName()}</td>
 * </tr>
 * * <tr>
 *    <td>TYPE</td>
 *    <td>{@link #getType()}</td>
 * </tr>
 * <tr>
 *    <td>PREV-SIBLING</td>
 *    <td>{@link #getPrevSibling()}</td>
 * </tr>
 * <tr>
 *    <td>NEXT-SIBLING</td>
 *    <td>{@link #getNextSibling()}</td>
 * </tr>
 * <tr>
 *    <td>PRIVATE-DATA</td>
 *    <td>{@link #getPrivateData()}, {@link #setPrivateData(character)} and
 *        {@link #setPrivateData(String)}
 *    </td>
 *  <tr>
 *    <td>Unknown status</td>
 *    <td>{@link #isUnknown()}</td>
 * </tr>
 * <tr>
 *    <td>Read only field error</td>
 *    <td>{@link #readOnlyError(String)}</td>
 * </tr>
 * </tr>
 * </table>
 * <p>
 * The abstract class also provides setters for the implemented attributes 
 * which are not defined inside the interfaces to be used for initializing of
 * those attributes
 */
public abstract class HandleChain
implements CommonHandleChain
{
   // TODO Extend this to other classes which need the common implemented methods
   /** Variable for holding the type attribute */
   private character type = new character();
   
   /** Variable for holding the name attribute*/
   private character name = new character();
   
   /** Variable for holding the privateData attribute*/
   private character privateData = new character();
   
   /** Variable for holding the PREV-SIBLING attribute */
   private handle prevSibling = null;
   
   /** Variable for holding the NEXT-SIBLING attribute*/
   private handle nextSibling = null;
   
   /** Variable for holding unknown state */
   private boolean isUnknown = true;
   
   /**
    * Get the type of this handle.
    * 
    * @return   See above.
    */
   public character getType()
   {
      return type;
   }
   
   /**
    * Sets the type of this handle.This method is outside interfaces and only 
    * meant for initializing this within the implementation.
    * 
    * @param type
    *        The handle type to be set.
    */
   public void setType(character type)
   {
      this.type = type;
   }
   
   /**
    * Sets the type of this handle.This method is outside interfaces and only 
    * meant for initializing this within the implementation.
    * 
    * @param type
    *        The handle type to be set.
    */
   public void setType(String type)
   {
      setType(new character(type));
   }

   /**
    * Show a specific error, as this read-only <code>attribute</code> was used
    * on the right-side of an assignment.
    *  
    * @param    attribute
    *           The read-only attribute.
    */
   public void readOnlyError(String attribute)
   {
      handle.readOnlyError(new handle(this), attribute);
   }

   /**
    * Reports if this object is unknown.  
    *
    * @return   <code>true</code> if object is unknown.
    */
   public boolean isUnknown()
   {
      return isUnknown;
   }
   
   /**
    * Sets whether this object is unknown.This method is outside interfaces and 
    * only meant for initializing this within the implementation.
    * 
    * @param isUnknown
    *        The handle type to be set.
    */
   public void setUnknown(boolean isUnknown)
   {
      this.isUnknown = isUnknown;
   }
   
   /**
    * Get the <code>name</code> attribute of handle.
    * 
    * @return   See above.
    */
   public character getName()
   {
      return name;
   }
   
   /**
    * Sets the name of this handle.This method is outside interfaces and only 
    * meant for initializing this within the implementation.
    * 
    * @param name
    *        The handle type to be set.
    */
   public void setName(character name)
   {
      this.name = name; 
   }
   
   /**
    * Sets the name of this handle.This method is outside interfaces and only 
    * meant for initializing this within the implementation.
    * 
    * @param name
    *        The handle type to be set.
    */
   public void setName(String name)
   {
      setName(new character(name)); 
   }

   /**
    * Get the <code>next-sibling</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public handle getNextSibling()
   {
      return nextSibling;
   }
   
   /**
    * Sets the NEXT-SIBLING attribute of this handle.This method is outside 
    * interfaces and only meant for initializing this within the implementation.
    * 
    * @param nextSibling
    *        The handle next sibling to be set.
    */
   public void setNextSibling(handle nextSibling)
   {
      this.nextSibling = nextSibling;
   }

   /**
    * Get the <code>prev-sibling</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public handle getPrevSibling()
   {
      return prevSibling;
   }
   
   /**
    * Sets the PREV-SIBLING attribute of this handle.This method is outside 
    * interfaces and only meant for initializing this within the implementation.
    * 
    * @param prevSibling
    *        The handle previous sibling to be set.
    */
   public void setPrevSibling(handle prevSibling)
   {
      this.prevSibling = prevSibling;
   }

   /**
    * Get the <code>private-data</code> attribute of this handle.
    * 
    * @return   See above.
    */
   public character getPrivateData()
   {
      return privateData;
   }

   /**
    * Set the <code>private-data</code> attribute of this handle.
    * 
    * @param    data
    *           The new private-data value. 
    */
   public void setPrivateData(character data)
   {
      this.privateData = data;
   }

   /**
    * Set the <code>private-data</code> attribute of this handle.
    * 
    * @param    data
    *           The new private-data value. 
    */
   public void setPrivateData(String data)
   {
      setPrivateData(new character(data));
   }
}
